""" prompt lib"""
from enum import Enum
from .prompts import Prompt,RichPrompt
from .prompts import Images
from .prompts import JudgeType,GenerateSchedule,Notes,ExtraText,Mermaid,GitHelp,Estimate_duration

from .prompts import ChatBox,MetaBox, MeetBox,memoryBox,DedaoExtract,Mermaid2canvas
class TemplateType(Enum):
    """ template type"""
    JUDGETYPE = 'JudgeType'
    GENERATE_SCHEDULE = 'GenerateSchedule'
    NOTES = 'Notes'
    EXTRATEXT = "ExtraText"
    GITHELP = "GitHelp"
    MERMAID = "Mermaid"
    ESTIMATE_DURATION = "Estimate_duration"
    RICH_IMAGES = "RichImages"
    ChatBox = "ChatBox"
    MetaBox = "MetaBox"
    MeetBox = "MeetBox"
    memoryBox = "memoryBox"
    DedaoExtract = "DedaoExtract"
    Mermaid2canvas = "Mermaid2canvas"

class Templates:
    """Template Factory"""
    def __new__(cls, template_type: TemplateType) -> Prompt|RichPrompt:
        assert template_type.value in [item.value for item in TemplateType]

        if template_type.value == 'JudgeType':
            return JudgeType()

        elif template_type.value == 'GenerateSchedule':
            # 定义提示模板字符串
            return GenerateSchedule()
        elif template_type.value == 'Notes':
            # 定义提示模板字符串
            return Notes()
        elif template_type.value == 'ExtraText':
            # 定义提示模板字符串
            return ExtraText()
        elif template_type.value == 'GitHelp':
            # 定义提示模板字符串
            return GitHelp()
        elif template_type.value == 'Mermaid':
            # 定义提示模板字符串
            return Mermaid()
        elif template_type.value == 'RichImages':
            # 定义提示模板字符串
            return Images()
        elif template_type.value == 'Estimate_duration':
            # 定义提示模板字符串
            return Estimate_duration()
        elif template_type.value == 'ChatBox':
            # 定义提示模板字符串
            return ChatBox()
        elif template_type.value == 'MetaBox':
            # 定义提示模板字符串
            return MetaBox()
        elif template_type.value == 'MeetBox':
            # 定义提示模板字符串
            return MeetBox()
        elif template_type.value == 'memoryBox':
            # 定义提示模板字符串
            return memoryBox()
        elif template_type.value == 'DedaoExtract':
            # 定义提示模板字符串
            return DedaoExtract()
        elif template_type.value == 'Mermaid2canvas':
            # 定义提示模板字符串
            return Mermaid2canvas()
        else:
            # 定义提示模板字符串
            return None



import re
import pandas as pd
import io


def extract_python_code(text: str)->str:
    """从文本中提取python代码

    Args:
        text (str): 输入的文本。

    Returns:
        str: 提取出的python文本
    """
    pattern = r'```python([\s\S]*?)```'
    matches = re.findall(pattern, text)
    return matches[0].replace('\n','')

def extract_type_code(text: str)->str:
    """从文本中提取python代码

    Args:
        text (str): 输入的文本。

    Returns:
        str: 提取出的python文本
    """
    pattern = r'```type([\s\S]*?)```'
    matches = re.findall(pattern, text)
    return matches[0].replace('\n','')


def dataframe_to_lm_string(df: pd.DataFrame, num_samples: int = 5, max_col_width: int = 50) -> str:
    """
    将 Pandas DataFrame 转换为一个对大模型友好的字符串表示。

    这个字符串包含数据描述、数据类型、前几行数据样本，
    旨在让大模型充分理解数据的结构和内容，以便执行后续操作。

    Args:
        df (pd.DataFrame): 需要转换的 Pandas DataFrame。
        num_samples (int): 要显示的数据样本行数。默认为 5。
        max_col_width (int): 显示数据样本时，每列的最大宽度，
                             超过部分将被截断并显示省略号。默认为 50。

    Returns:
        str: 包含数据描述和样本的字符串，适合作为大模型的输入。
    """
    if not isinstance(df, pd.DataFrame):
        raise TypeError("输入必须是一个 Pandas DataFrame。")

    output_string = io.StringIO()

    # 1. 添加数据描述和总览
    output_string.write("# 数据集概览\n\n")
    output_string.write(f"这是一个包含 {len(df)} 行， {len(df.columns)} 列的数据集。\n")
    output_string.write("下面是关于数据集列的描述：\n\n")

    # 2. 添加列信息（列名、数据类型、非空数量）
    output_string.write("## 列信息\n\n")
    output_string.write("| 列名 | 数据类型 | 非空数量 |\n")
    output_string.write("|---|---|---|\n")
    for col in df.columns:
        dtype = df[col].dtype
        non_null_count = df[col].count()
        output_string.write(f"| {col} | {dtype} | {non_null_count} |\n")
    output_string.write("\n")

    # 3. 添加数据样本
    output_string.write(f"## 数据样本 (前 {num_samples} 行)\n\n")
    if len(df) > 0:
        # 使用 to_csv 转换为 CSV 格式字符串，方便模型解析
        # 同时处理列宽，避免过长影响可读性
        sample_df = df.head(num_samples).copy()
        for col in sample_df.columns:
            # 将非字符串类型转换为字符串，方便截断
            sample_df[col] = sample_df[col].astype(str).str.slice(0, max_col_width) + sample_df[col].astype(str).apply(lambda x: '...' if len(x) > max_col_width else '')

        sample_csv = sample_df.to_csv(index=False)
        output_string.write("```csv\n") # 使用CSV代码块，明确格式
        output_string.write(sample_csv)
        output_string.write("```\n")
    else:
        output_string.write("数据集为空，无法显示样本。\n")

    return output_string.getvalue()



import re
import logging

# 配置日志记录
logging.basicConfig(level=logging.INFO, format='%(asctime)s - %(levelname)s - %(message)s')

def extract_python_code(text: str) -> str:
    """从文本中提取python代码
    Args:
        text (str): 输入的文本。
    Returns:
        str: 提取出的python文本
    """
    pattern = r'```python([\s\S]*?)```'
    matches = re.findall(pattern, text)
    if matches:
        return matches[0].strip() # 添加strip()去除首尾空白符
    else:
        logging.warning("未能在文本中提取到Python代码块。")
        return "" # 返回空字符串或抛出异常，此处返回空字符串

def work(x="帮我将 日期 2025/12/03 向前回退12天", **kwargs):
    try:
        from llmada.core import BianXieAdapter
        params = locals()

        # 优化提示信息，只在kwargs不为空时添加入参信息
        prompt_user_part = f'{x}'
        if kwargs:
            prompt_user_part += f' 入参 {params["kwargs"]}'

        bx = BianXieAdapter()
        result = bx.product(prompt=f"""
# 用户输入案例：
# [用户指令和输入参数的组合，例如：帮我将 日期 2025/12/03 向前回退12天 入参 data_str = "2025/12/03"]
#
# 根据用户输入案例，自动识别用户指令，提取所有“入参”信息及其名称和值。
# 编写一个 Python 函数，函数名称固定为 `Function`。
# 函数的输入参数应严格按照识别出的“入参”名称和类型来定义。
# 函数只包含一个定义，不依赖外部库或复杂结构。
# 函数应根据用户指令实现对应功能。
# 代码应简洁且易于执行。
# 输出格式应为完整的 Python 函数定义，包括文档字符串（docstring）。
{prompt_user_part}
""")
        xx = extract_python_code(result)

        if not xx:
            logging.error("提取到的Python代码为空，无法执行。")
            return None # 返回None或抛出异常

        runs = xx + '\n' + f'result = Function(**{params["kwargs"]})'
        logging.info(f"即将执行的代码：\n{runs}")

        rut = {'result': ''}
        # 使用exec执行代码，并捕获可能的错误
        try:
            exec(runs, globals(), rut) # 将globals()作为全局作用域，避免依赖外部locals()
        except Exception as e:
            logging.error(f"执行动态生成的代码时发生错误: {e}")
            return None # 返回None或抛出异常

        return rut.get('result')

    except ImportError:
        logging.error("无法导入 llmada.core，请确保已安装相关库。")
        return None
    except Exception as e:
        logging.error(f"在 work 函数中发生未知错误: {e}")
        return None

# 示例调用
if __name__ == "__main__":
    pass
    # 示例1：带参数
    # result1 = work(x="帮我将 日期 2025/12/03 向前回退12天", data_str="2025/12/03", days=12)
    # print(f"示例1 执行结果: {result1}")

    # 示例2：不带参数
    # result2 = work(x="帮我计算 1加1")
    # print(f"示例2 执行结果: {result2}")
